﻿using System;

namespace UMK.Kawiarnia
{
    //component
    public abstract class Napój
    {
        public abstract decimal Cena();
        public void Objętość() //funckja nie podlegająca zmianom podczas dekoracji
        {
            Console.WriteLine("\nObjetosc: 330 ml");
        }
    }

    //concrete components
    class Herbata : Napój
    {
        public override decimal Cena()
        {
            return 6.70M;
        }

        public override string ToString()
        {
            return "Herbata";
        }
    }

    class Kawa : Napój
    {
        public override decimal Cena()
        {
            return 7.90M;
        }

        public override string ToString()
        {
            return "Kawa";
        }
    }

    //decorator
    abstract class NapójZDodatkiem : Napój
    {
        protected Napój napój;

        public NapójZDodatkiem(Napój napój)
        {
            this.napój = napój;
        }
    }

    //concrete decorator
    class NapójZŁyżeczkąCukru : NapójZDodatkiem
    {
        public NapójZŁyżeczkąCukru(Napój napój)
            :base(napój) 
        { }

        public override decimal Cena()
        {
            return napój.Cena() + 1M;
        }

        public override string ToString()
        {
            return napój.ToString() + " z łyżeczką cukru";
        }
    }

    class NapójZCytryną : NapójZDodatkiem
    {
        uint liczbaPlasterków;

        public NapójZCytryną(Napój napój, uint liczbaPlasterków)
            :base(napój)
        {
            this.liczbaPlasterków = liczbaPlasterków;
        }

        public override decimal Cena()
        {
            return napój.Cena() + liczbaPlasterków * 1.5M;
        }

        public override string ToString()
        {
            return napój.ToString() +$" z {liczbaPlasterków} plastrami cytryny";
        }
    }

    class NapójZeSzczyptąCynamonu : NapójZDodatkiem
    {
        public NapójZeSzczyptąCynamonu(Napój napój)
            : base(napój)
        { }

        public override decimal Cena()
        {
            return napój.Cena() + 0.5M;
        }

        public override string ToString()
        {
            return napój.ToString() + " ze szczyptą cynamonu";
        }
    }
}
